@@ -1,5 +1,5 @@
 /* defs.h -- data types and declarations.
-   Copyright (C) 1990-2019 Free Software Foundation, Inc.
+   Copyright (C) 1990-2022 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -138,14 +138,6 @@ struct perm_val
   mode_t val[2];
 };
 
-/* dir_id is used to support loop detection in oldfind.c
- */
-struct dir_id
-{
-  ino_t ino;
-  dev_t dev;
-};
-
 /* samefile_file_id is used to support the -samefile test.
  */
 struct samefile_file_id
@@ -327,7 +319,7 @@ struct predicate
     struct samefile_file_id samefileid; /* samefile */
     bool types[FTYPE_COUNT];	/* file type(s) */
     struct format_val printf_vec; /* printf fprintf fprint ls fls print0 fprint0 print */
-    security_context_t scontext; /* security context */
+    char *scontext; /* security context */
   } args;
 
   /* The next predicate in the user input sequence,
@@ -346,14 +338,13 @@ struct predicate
   const struct parser_table* parser_entry;
 };
 
-/* oldfind.c, ftsfind.c */
+/* ftsfind.c */
 bool is_fts_enabled(int *ftsoptions);
 
 /* find library function declarations.  */
 
 /* find global function declarations.  */
 
-/* oldfind.c */
 /* SymlinkOption represents the choice of
  * -P, -L or -P (default) on the command line.
  */
@@ -363,7 +354,6 @@ enum SymlinkOption
     SYMLINK_ALWAYS_DEREF,	/* Option -L */
     SYMLINK_DEREF_ARGSONLY	/* Option -H */
   };
-extern enum SymlinkOption symlink_handling; /* defined in oldfind.c. */
 
 void set_follow_state (enum SymlinkOption opt);
 void cleanup(void);
@@ -517,7 +507,6 @@ void nonfatal_nontarget_file_error (int errno_value, const char *name);
 
 int process_leading_options (int argc, char *argv[]);
 void set_option_defaults (struct options *p);
-void error_severity (int level);
 
 # if 0
 #  define apply_predicate(pathname, stat_buf_ptr, node)	\
@@ -529,8 +518,7 @@ bool apply_predicate(const char *pathname, struct stat *stat_buf, struct predica
 # define pred_is(node, fn) ( ((node)->pred_func) == (fn) )
 
 
-/* oldfind.c. */
-int get_info (const char *pathname, struct stat *p, struct predicate *pred_ptr);
+/* util.c. */
 bool following_links (void);
 bool digest_mode (mode_t *mode, const char *pathname, const char *name, struct stat *pstat, bool leaf);
 bool default_prints (struct predicate *pred);
@@ -626,7 +614,7 @@ struct options
   int regex_options;
 
   /* function used to get file context */
-  int (*x_getfilecon) (int, const char *, security_context_t *);
+  int (*x_getfilecon) (int, const char *, char **);
 
   /* Optimisation level.  One is the default.
    */
@@ -636,6 +624,12 @@ struct options
   /* How should we quote filenames in error messages and so forth?
    */
   enum quoting_style err_quoting_style;
+
+  /* Read starting points from FILE (instead of argv).  */
+  const char *files0_from;
+
+  /* True if actions like -ok, -okdir need a user confirmation via stdin.  */
+  bool ok_prompt_stdin;
 };
 
 
