@@ -1,5 +1,5 @@
 /* tree.c -- helper functions to build and evaluate the expression tree.
-   Copyright (C) 1990-2019 Free Software Foundation, Inc.
+   Copyright (C) 1990-2022 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -128,11 +128,19 @@ get_expr (struct predicate **input,
       break;
 
     case CLOSE_PAREN:
+      if (prev_pred == NULL)
+	{
+	  /* Happens with e.g. "find -files0-from - ')' -print" */
+	  die (EXIT_FAILURE, 0,
+	       _("invalid expression: expected expression before closing parentheses '%s'."),
+	       this_pred->p_name);
+	}
+
       if ((UNI_OP == prev_pred->p_type
 	  || BI_OP == prev_pred->p_type)
 	  && !this_pred->artificial)
 	{
-	  /* e.g. "find \( -not \)" or "find \( -true -a \" */
+	  /* e.g. "find \( -not \)" or "find \( -true -a \)" */
 	  die (EXIT_FAILURE, 0,
 	       _("expected an expression between '%s' and ')'"),
 	       prev_pred->p_name);
@@ -180,6 +188,12 @@ get_expr (struct predicate **input,
       *input = (*input)->pred_next;
       if ( (*input)->p_type == CLOSE_PAREN )
 	{
+	  if (prev_pred->artificial)
+	    {
+	      die (EXIT_FAILURE, 0,
+		   _("invalid expression: expected expression before closing parentheses '%s'."),
+		   (*input)->p_name);
+	    }
 	  die (EXIT_FAILURE, 0,
 	       _("invalid expression; empty parentheses are not allowed."));
 	}
@@ -661,7 +675,7 @@ do_arm_swaps (struct predicate *p)
      occurs according to the p_cost member of each predicate (which
      reflects the performance cost of the test).  The ordering also
      bears in mind whether these operations are more likely to succeed
-     or fail.  When evauating a chain of OR conditions, we prefer
+     or fail.  When evaluating a chain of OR conditions, we prefer
      tests likely to succeed at the front of the list.  For AND, we
      prefer tests likely to fail at the front of the list.
 
