@@ -3,7 +3,7 @@
 # Between FINDUTILS_4_3_1-1 and 4.6, find crashed on some platforms.
 # See Savannah bug #52220.
 
-# Copyright (C) 2017-2019 Free Software Foundation, Inc.
+# Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,15 +18,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-. "${srcdir=.}/tests/init.sh"
-print_ver_ find oldfind
+. "${srcdir=.}/tests/init.sh"; fu_path_prepend_
+print_ver_ find
 
-# Exercise both find executables.
-for exe in find oldfind; do
-  rm -f out err || framework_failure_
-  returns_ 1 "$exe" -D >/dev/null 2> err || fail=1
-  grep -F "find: Missing argument after the -D option." err \
-    || { cat err; fail=1; }
-done
+returns_ 1 find -D >/dev/null 2> err || fail=1
+grep -F "find: Missing argument after the -D option." err \
+  || { cat err; fail=1; }
 
 Exit $fail
