@@ -2,7 +2,7 @@
 # Test that find -name treats the unquoted '[' argument literally.
 # See Savannah bug #32043.
 
-# Copyright (C) 2011-2019 Free Software Foundation, Inc.
+# Copyright (C) 2011-2022 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-. "${srcdir=.}/tests/init.sh"
-print_ver_ find oldfind
+. "${srcdir=.}/tests/init.sh"; fu_path_prepend_
+print_ver_ find
 
 # Prepare a file named '['.
 touch '[' || framework_failure_
@@ -27,7 +27,4 @@ echo './[' > exp || framework_failure_
 find -name '[' -print > out || fail=1
 compare exp out || fail=1
 
-oldfind -name '[' -print > out2 || fail=1
-compare exp out2 || fail=1
-
 Exit $fail
