@@ -1,7 +1,7 @@
 #!/bin/sh
 # Test -printf with the \c escape character.
 
-# Copyright (C) 2011-2019 Free Software Foundation, Inc.
+# Copyright (C) 2011-2022 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,20 +16,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-. "${srcdir=.}/tests/init.sh"
-print_ver_ find oldfind
+. "${srcdir=.}/tests/init.sh"; fu_path_prepend_
+print_ver_ find
 
 echo 'hello^.^world' > exp || framework_failure_
 
-for executable in oldfind find; do
-  rm -f out || framework_failure_
-  $executable . -maxdepth 0 \
-    -printf 'hello^\cthere' \
-    -exec printf %s {} \; \
-    -printf '^world\n' \
-    > out || fail=1
+find . -maxdepth 0 \
+  -printf 'hello^\cthere' \
+  -exec printf %s {} \; \
+  -printf '^world\n' \
+  > out || fail=1
 
-  compare exp out || fail=1
-done
+compare exp out || fail=1
 
 Exit $fail
